from yahoofantasy.api.attr import APIAttr
from yahoofantasy.api.parse import from_response_object


class Standings():

    def __init__(self):
        self._standings = list()

    def update_team_references(self, league):
        """ Update each standings item with their league team reference """
        for standings_team in self._standings:
            setattr(standings_team,
                    '_team_obj',
                    league.get_team(standings_team.team_key))

    @staticmethod
    def from_response(resp):
        standings = Standings()
        for standing_result in resp:
            standings._standings.append(from_response_object(
                APIAttr(), standing_result))
        return standings



import json
from os.path import join, dirname
from yahoofantasy.api.fetch import make_request
from yahoofantasy.api.parse import parse_response


def generate_stats(game, token):
    """ Generate a python file with a stat mapping

    Args:
        game (string): The game to generate for (e.g. nfl, mlb)
        token: An access token to talk to the API

    Returns:
        None - writes a file called mlb.py with a stats export
    """
    stats_resp = parse_response(make_request(
        'game/{}/stat_categories'.format(game), token=token))
    stats = stats_resp['fantasy_content']['game']['stat_categories']['stats']['stat']  # noqa E501
    mapping = {}
    for stat in stats:
        mapping[stat['stat_id']['$']] = {
            'name': stat['name']['$'],
            'display': stat['display_name']['$'],
            'order': int(stat['sort_order']['$']),
        }
    with open(join(dirname(__file__), '{}.py'.format(game)), 'w+') as f:
        f.write('stats=')
        json.dump(mapping, f)


if __name__ == '__main__':
    import argparse
    parser = argparse.ArgumentParser()
    parser.add_argument("game")
    parser.add_argument("token")
    args = parser.parse_args()
    generate_stats(args.game, args.token)



from yahoofantasy import logger
import requests

YURL = "https://fantasysports.yahooapis.com/fantasy/v2"
TOKEN = "wX8SGPaZtV9te.OpyMOxy_Np2VtNQAlz3oSlfzsNH6KziRyfgoMKi0KBrihXuXJhngjr05561bMFqeoFFJVXj4o1Cwk4jBe1H_Ul6P0HrFJUkuK5i2u4KXSn9.g16JjANMuxd47nMveTsxFvGqJ6JD.ZPQ0hz6c5jnRLCWtwd3LGOZJhVmcqnOpfMOnASfL8gaHOfbuRV0ESVT8qodVdRzSpEzDfTcWUdLlt1J1WbU9MqHq6iZ6F5FTM9errZV7i4DN0KqT_r73gdkChv5G3gduHPVuog80sGHTPmeCsbijPazJTtpI2YfSMWhxxwToAz4DeMJIqkpaWIJ2Dduj0.ahvBFsVIw_x4v_g7yDHF2Xzd4vU0DwHFhI8.SilHS.oEEN14j189__cZGM.uA2MfeqLHSM844lQz0tcUNFlkJSVScOXH_FkCbEr02ZCUzQggJSjG6d9z0VeGH29pQ9EcQPVMbiwUPzK.QMUofMmES.fFvqHz8l2qpDe.OMTd28rliWyEE5qJN2oSoojCx.di6h.vxddgN_SHSkhoW0M3ErxaIpOncmol93ajysCS7zYIItWc2qu3o3FCeZsOMqMU.5gm3B5_6jvTnAYWgtdjh55GOd1RhCb4bFb5Rdw1ZjE6fbzsSn.r38R8RsPgr6eEZfz2iK7QCskeJa3XEaiol_fcuAa1UBMdUfKzUsr_fGWAJR3nlN.oi3Fc5tFuPB_lzDHIwXWoXIRPqgzpP95hfyNmU7MYyeAjJQ_wZVr4QjC_mmoBW4a4Q_yTxWHRWk9vArC7vUCWyM2JOBC7ATBIVUspt5mT2bQaiUubLVG090tJylUIYUwPCb3SkxTlE.Ym5twQAGaBaFdGglabDmMcyHlXi0V1AOxJsK32Y6g_0C4U.lQh5pnRy0nRc_e63EU8RDV6w08XcfYXNvTNgVWEm.VMYSF5EyTKwPvqvGsbsCFNH29yeAW4QHnuEerPs2akM2K3v.SInADYL5q_BXIAW9aYDBM445TXw7PJD1DLgb7KspDbisGOn3JlV3uSjcavJycBnRu7uf9CNZQomsu4gqGNMQKA5nvqt6l.95n"  # noqa


def make_request(url, token=TOKEN, league=False, **kwargs):
    if league:
        url = "league/{}/{}".format(league, url)
    logger.debug("Making request to {}".format(url))
    resp = requests.get(
        "{}/{}".format(YURL, url),
        headers={"Authorization": "Bearer {}".format(token)}
    )

    try:
        resp.raise_for_status()
    except:
        logger.exception("Bad response status ({}) for request".format(
            resp.status_code))
        raise
    return resp.text





class League():

    def __init__(self):
        self.teams = list()
        self.players = list()
        self.standings = None
        self.weeks = list()

    def get_team(self, team_key):
        return next((t for t in self.teams if t.team_key == team_key), None)



from yahoofantasy import logger
from os.path import isfile
import pickle
from time import time


def get_persistence_filename(persist_key):
    return "{}.yahoo.data".format(persist_key)


def save_league_to_persistence(league, persist_key):
    with open(get_persistence_filename(persist_key), 'wb') as fp:
        obj_to_save = {
            "time": int(time())
        }

        # Save whatever we can
        try:
            obj_to_save["league"] = league._raw
        except AttributeError:
            pass

        try:
            obj_to_save["teams"] = league._teams_raw
        except AttributeError:
            pass

        try:
            obj_to_save["players"] = league._players_raw
        except AttributeError:
            pass

        try:
            obj_to_save["standings"] = league.standings._raw
        except AttributeError:
            pass

        try:
            obj_to_save["weeks"] = {
                week_num + 1: league.weeks[week_num]._raw
                for week_num in range(len(league.weeks))}
        except AttributeError:
            pass

        pickle.dump(obj_to_save, fp)


def fetch_league_from_persistence(persist_key, ttl=1800):
    if not persist_key:
        return None

    filename = get_persistence_filename(persist_key)

    if not isfile(filename):
        logger.info("Persistence file not found")
        return None

    with open(get_persistence_filename(persist_key), 'rb') as fp:
        try:
            persisted_data = pickle.load(fp)
        except Exception:
            logger.exception("Could not load persistence data")
            return None

    # If we have persisted data and it's within the TTL, return it
    if not persisted_data:
        logger.warning("No persistence data found in file")
        return None

    if persisted_data.get('time') + ttl < time():
        logger.info("Persistence data expired, ignoring")
        return None

    return persisted_data



from unittest import TestCase
from unittest.mock import patch
from yahoofantasy.api.connection import Connection
# from .sample_responses import make_test_request


class TestConnection(TestCase):

    @patch('yahoofantasy.api.fetch.make_request', return_value={})
    @patch('yahoofantasy.api.parse.from_response_object')
    def test_fetch_no_persist(self, _, mock_request):
        con = Connection(persist=None)
        con.fetch()
        import pdb
        pdb.set_trace()  # XXX BREAKPOINT
        pass



import re
from xmljson import badgerfish as bf
from xml.etree.ElementTree import fromstring
from yahoofantasy import logger
from yahoofantasy.api.attr import APIAttr


def parse_response(resp):
    """ Parse the raw XML response into OrderedDicts """
    # Remove the namespace from the first XML element
    # Yahoo adds this namespace and the xml to json library
    # prefixes it on every json record, which is very annoying
    non_ns_text = re.sub(
        r'<fantasy_content[^>]*>', '<fantasy_content>', resp)

    # Return a JSON representation of the XML data returned from Yahoo
    return bf.data(fromstring(non_ns_text))


def get_value(val):
    """ Get the value to set based on a value from an XML response """

    # If it's a list we need one for each
    if isinstance(val, list):
        return [get_value(sub_val) for sub_val in val]

    if not isinstance(val, dict):
        logger.warn('Getting value of non dict...weird?')
        return val

    if '$' in val:
        if len(val) > 1:
            logger.warn('Value has a $ but other stuff too')
        return val['$']

    return from_response_object(APIAttr(), val)


def as_list(val):
    if isinstance(val, list):
        return val
    return [val]


def from_response_object(obj, resp):
    """ Sets the attributes on obj based on resp """
    if not isinstance(resp, dict):
        raise RuntimeError(
            "Cannot parse response object that isn't dict")

    for attr in resp:
        setattr(obj, attr, get_value(resp[attr]))

    return obj



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

reqs = [
    'requests',
    'xmljson',
]

setuptools.setup(
    name="yahoofantasy",
    version="0.0.2",
    author="Matt Dodge",
    author_email="matt@dodge.com",
    description="An SDK for the Yahoo! Fantasy Sports API",
    long_description=long_description,
    long_description_content_type="text/markdown",
    install_requires=reqs,
    url="https://github.com/mattdodge/yahoofantasy",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)



from yahoofantasy import logger
from yahoofantasy.api.fetch import make_request
from yahoofantasy.api.parse import from_response_object, parse_response
from yahoofantasy.util.persistence import fetch_league_from_persistence
from yahoofantasy.league import League
from yahoofantasy.team import Team
from yahoofantasy.player import Player
from yahoofantasy.week import Week
from yahoofantasy.standings import Standings


class Connection():

    def __init__(self, league_id, persist=None):
        self.__persist_key = persist
        self._league_id = league_id
        self.league = League()

    def fetch(self, persist_ttl=1800):
        """ Fetch all information about this league """
        persisted_raw = fetch_league_from_persistence(
            self.__persist_key, ttl=persist_ttl)

        if not persisted_raw:
            persisted_raw = {}

        league = League()

        self.__parse_league(league, raw=persisted_raw.get('league'))
        self.__parse_teams(league, raw=persisted_raw.get('teams'))
        self.__parse_players(league, raw=persisted_raw.get('players'))
        # self.__parse_standings(league, raw=persisted_raw.get('standings'))
        # self.__parse_weeks(league, raw=persisted_raw.get('weeks'))

        return league

    def __parse_league(self, league, raw=None):
        if raw is None:
            raw = make_request('', league=self._league_id)
        league._raw = raw
        parsed = parse_response(raw)
        from_response_object(league, parsed['fantasy_content']['league'])

    def __parse_teams(self, league, raw=None):
        if raw is None:
            raw = make_request('teams', league=self._league_id)
        league._teams_raw = raw
        parsed = parse_response(raw)
        for team in parsed['fantasy_content']['league']['teams']['team']:
            t = Team.from_response(team)
            league.teams.append(t)

    def __parse_players(self, league, raw=None):
        if raw is None:
            raw = make_request('players', league=self._league_id)
        league._players_raw = raw
        parsed = parse_response(raw)
        for player in parsed['fantasy_content']['league']['players']['player']:
            p = Player.from_response(player)
            league.players.append(p)

    def __parse_standings(self, league, raw=None):
        if raw is None:
            raw = make_request('standings', league=self._league_id)
        parsed = parse_response(raw)
        standings = Standings.from_response(
            parsed['fantasy_content']['league']['standings']['teams']['team'])
        standings._raw = raw
        league.standings = standings
        league.standings.update_team_references(league)

    def __parse_weeks(self, league, raw=None):
        for week_num in range(league.start_week, league.end_week + 1):
            if not raw or not raw.get(week_num):
                raw_week = make_request('scoreboard;week={}'.format(week_num),
                                        league=self._league_id)
            else:
                raw_week = raw.get(week_num)

            parsed = parse_response(raw_week)
            week = Week.from_response(
                parsed['fantasy_content']['league']['scoreboard']['matchups'])

            week._raw = raw_week
            league.weeks.append(week)



from yahoofantasy.matchup import Matchup
from yahoofantasy.api.parse import from_response_object


class Week():

    def __init__(self):
        self._matchups = list()

    @staticmethod
    def from_response(resp):
        week = Week()

        # If no matchup has been scheduled yet (i.e., playoffs) we ignore
        if 'matchup' not in resp:
            return week

        for matchup in resp['matchup']:
            matchup_obj = Matchup()
            week._matchups.append(from_response_object(
                matchup_obj, matchup))
        return week



def make_test_request(url, token, league=None):
    if url == '':
        return get_test_league_response()
    elif url == 'teams':
        return get_test_teams_response()
    elif url == 'standings':
        return get_test_standings_response()
    elif url.startswith('scoreboard'):
        return get_test_scoreboard_response()

    raise RuntimeError("Invalid Request Made")


def __wrap_response(obj):
    return {
        'fantasy_content': {
            'league': obj
        }
    }


def get_test_league_response():
    return __wrap_response({
        'start_week': 1,
        'end_week': 3,
    })


def __get_test_team(team_id):
    return {
        'name': 'Team {}'.format(team_id),
        'team_key': team_id,
    }


def get_test_teams_response():
    return __wrap_response({
        'teams': {
            'team': [
                __get_test_team(1),
                __get_test_team(2),
                __get_test_team(3),
                __get_test_team(4),
            ]
        }
    })


def get_test_standings_response():
    return __wrap_response({
        'standings': {
            'teams': {
                'team': [
                    __get_test_team(1),
                    __get_test_team(2),
                    __get_test_team(3),
                    __get_test_team(4),
                ]
            }
        }
    })


def get_test_scoreboard_response():
    return __wrap_response({
        'scoreboard': {
            'matchups': {
                'matchup': [
                    {'teams': {
                        'team': [
                            __get_test_team(1),
                            __get_test_team(2),
                        ]
                    }},
                    {'teams': {
                        'team': [
                            __get_test_team(3),
                            __get_test_team(4),
                        ]
                    }},
                ]
            }
        }
    })
